#include <fs_attr.h>
#include <malloc.h>
#include <stdio.h>

#include <Node.h>
#include <Alert.h>
#include <File.h>
#include <Application.h>
#include <Button.h>
#include <StringView.h>
#include <TextControl.h>
#include <StringView.h>
#include <ScrollView.h>
#include <MenuItem.h>
#include <MenuField.h>
#include <Query.h>
#include <Volume.h>
#include <Directory.h>
#include <Path.h>
#include <FindDirectory.h>
#include <E-mail.h>

#include "FilelistItem.h"
#include "FilelistView.h"
#include "MailToWindow.h"



CMailToWindow::CMailToWindow()
	: BWindow( BRect( 200, 100, 480, 300 ), "Send To", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE )
{
	Build();
}

CMailToWindow::CMailToWindow( float x, float y )
	: BWindow( BRect( x, y, x+200, y+200 ), "Send To", B_TITLED_WINDOW, 0 )
{
	Build();
}

CMailToWindow::~CMailToWindow()
{
	CFilelistItem *item;
	for( int32 i = 0; ( item = ( CFilelistItem * )_Listview->ItemAt( i ) ); i++ )
		delete item;
}

bool CMailToWindow::QuitRequested()
{
	if( isApp )
		be_app->PostMessage( B_QUIT_REQUESTED );
	return true;
}

void CMailToWindow::Build()
{
	isApp = false;

	BMenuField *field;
	BRect r = Bounds();
	r.InsetBy( -1, -1 );
	
	_PanelView = new CPanelView( Bounds(), "back", B_FOLLOW_ALL, B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE | B_DRAW_ON_CHILDREN );
	AddChild( _PanelView );

	_SendToMenu = new BPopUpMenu( "To:" );
	_SendToMenu->SetRadioMode( false );

	BuildMenu();

	field = new BMenuField( BRect( 10, 6, 14, 22 ), "", NULL, _SendToMenu );
	_PanelView->AddChild( field );

	BStringView *v = new BStringView( BRect( 6, 6, 6 + 20, 22 ), "name", "To:" );
	v->SetAlignment( B_ALIGN_RIGHT );
	//_PanelView->AddChild( v );	

	_Email = new BTextControl( BRect( 48 , 8, 270, 22 ), "email", NULL, "", new BMessage('emil') );
	_PanelView->AddChild( _Email );

	_Subject = new BTextControl( BRect( 48 , 32, 270, 38 ), "subjectl", NULL, "", new BMessage('subj') );
	_PanelView->AddChild( _Subject );
	_Subject->SetTarget( this );

	_Listview = new CFilelistView( BRect( 53, 58, 267-B_V_SCROLL_BAR_WIDTH, 160 ), "list" );
	BScrollView *sv = new BScrollView( "scroll", _Listview, B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, false, true );
	_PanelView->AddChild( sv );

	_SendButton = new BButton( BRect( 199, 167, 199+24+48, 167+24 ), "sendb", "Send", new BMessage( SEND_BUTTON ) );
	_PanelView->AddChild( _SendButton );
	_SendButton->SetTarget( this );

	BStringView *tv = new BStringView( BRect( 6, 32, 44, 48 ), "text", "Subject:" );
	_PanelView->AddChild( tv );

}

void CMailToWindow::BuildMenu()
{
	BMenuItem 	*item;
	BEntry		entry;
	BPath 		path;
   	BDirectory	dir;
	BMessage	*msg;
	BFile		file;
	attr_info	ainfo;
	char 		*txt;		
	char		name[B_FILE_NAME_LENGTH]; 

	entry_ref	e;

	find_directory( B_USER_DIRECTORY, &path );
	path.Append( "people" );

   	dir.SetTo( path.Path() );

	txt = (char *)malloc(1);
	txt[0] = 0;

	while( dir.GetNextEntry( &entry ) != B_ENTRY_NOT_FOUND )
	{
		if (entry.InitCheck() == B_OK) 
		{
			entry.GetPath( &path );
			entry.GetName( name );
			if (path.InitCheck() == B_OK) 
			{
				file.SetTo( &entry, B_READ_ONLY );
				if( file.GetAttrInfo( "META:email", &ainfo ) == B_NO_ERROR )
				{
					if( ainfo.size > 1 )
					{
						txt = (char *)realloc( txt, ainfo.size );
						file.ReadAttr( "META:email", B_STRING_TYPE, 0, txt, ainfo.size );
						msg = new BMessage( MENU_PERSON );
						msg->AddString( "email", txt );
						item = new BMenuItem( name, msg );
						_SendToMenu->AddItem( item );
					}
				}
			}
		}
	}
}

void CMailToWindow::MessageReceived( BMessage *msg )
{
	switch( msg->what )
	{
		case MENU_PERSON:
		{
			const char *c = msg->FindString( "email" );
			_Email->SetText( c );
		}
			break;
		case B_SIMPLE_DATA:
		{
			entry_ref ref;
			for ( int count = 0; msg->FindRef( "refs", count, &ref ) == B_NO_ERROR; count ++ ) 
			{
				AddRef( &ref );
			}
		}
			break;		
		case SEND_BUTTON:
		{
			BMailMessage *m = new BMailMessage();
			if( strlen( _Email->Text() ) < 3 )
			{
				(new BAlert( "", "Bad E-mail adress", "OK" ) )->Go();
				_Email->MakeFocus();
				break;
			}
			if( strlen( _Subject->Text() ) == 0 )
			{
				(new BAlert( "", "No subject", "OK" ) )->Go();
				_Subject->MakeFocus();
				break;
			}	
			m->AddHeaderField( B_MAIL_TO, _Email->Text() );
			m->AddHeaderField( B_MAIL_SUBJECT, _Subject->Text() );
			CFilelistItem *tmp;
			for( int i = 0; i < _Listview->CountItems(); i++ )
			{
				tmp = ( CFilelistItem * )_Listview->ItemAt( i );
				m->AddEnclosure( tmp->Ref() );
			}
			m->Send( true );
			PostMessage( B_QUIT_REQUESTED );
		}
			break;
		default:	
			BWindow::MessageReceived( msg );
	}
}

void CMailToWindow::AddRef( entry_ref *ref )
{
	BNode node;
	node.SetTo( ref );
	if ( node.InitCheck( ) == B_OK ) 
	{
		if( !node.IsDirectory() )
		{
			bool inlist = false;
			CFilelistItem *tmp;
			CFilelistItem *item = new CFilelistItem( ref );

			Looper()->Lock();
			for( int i = 0; i < _Listview->CountItems(); i++ )
			{
				tmp = ( CFilelistItem * )_Listview->ItemAt( i );
				if( *tmp == *item )
					inlist = true;
			}
			if( !inlist )
				_Listview->AddItem( item );
			Looper()->Unlock();
		}
		else	
		{
			char t[1024];
			BEntry e;
			BPath p;
			e.SetTo( ref );
			e.GetPath( &p );
			sprintf(t, "Can not add folder or disk '%s'. ", p.Path() );	
			( new BAlert( "oso", t, "Dang!" ) )->Go();
		}
	}
}


rgb_color rgb( uint8 r, uint8 g, uint8 b )
{
	rgb_color c;
	c.red = r;
	c.green = g;
	c.blue = b;
	return c;
}

